package SHARYANTO::Utils;

our $VERSION = '0.29'; # VERSION

1;
# ABSTRACT: SHARYANTO's temporary namespace for various routines


=pod

=head1 NAME

SHARYANTO::Utils - SHARYANTO's temporary namespace for various routines

=head1 VERSION

version 0.29

=head1 FAQ

=head2 What is this?

This distribution is a heterogenous collection of modules that will eventually
have their own proper distributions, but do not yet because they are not ready
for some reason or another. For example: alpha quality code, code not yet
properly refactored, there are still no tests and/or documentation, I haven't
decided on a proper name yet, etc.

I put it on CPAN because some of my other modules (and scripts) depend on it.
And I always like to put as much of my code in functions and modules (as opposed
to scripts) as possible, for better reusability.

You are free to use this, but beware that things might get moved around without
prior warning.

I assure you that this is not a vanity distribution :-)

=head1 DESCRIPTION


This module has L<Rinci> metadata.

=head1 FUNCTIONS


None are exported by default, but they are exportable.

=head1 AUTHOR

Steven Haryanto <stevenharyanto@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Steven Haryanto.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut


__END__

